@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2020, The PurpleI2P Project
+* Copyright (c) 2013-2022, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -45,6 +45,7 @@ namespace i2p
 namespace win32
 {
 	DWORD g_GracefulShutdownEndtime = 0;
+	bool g_isWinService;
 
 	static void ShowPopupMenu (HWND hWnd, POINT *curpos, int wDefaultItem)
 	{
@@ -144,35 +145,34 @@ namespace win32
 		s << bytes << " Bytes\n";
 	}
 
-	static void ShowNetworkStatus (std::stringstream& s, RouterStatus status)
+	static void ShowNetworkStatus (std::stringstream& s, RouterStatus status, bool testing)
 	{
 		switch (status)
 		{
 			case eRouterStatusOK: s << "OK"; break;
-			case eRouterStatusTesting: s << "Test"; break;
 			case eRouterStatusFirewalled: s << "FW"; break;
 			case eRouterStatusUnknown: s << "Unk"; break;
 			case eRouterStatusProxy: s << "Proxy"; break;
 			case eRouterStatusMesh: s << "Mesh"; break;
-			case eRouterStatusError:
-			{
-				s << "Err";
-				switch (i2p::context.GetError ())
-				{
-					case eRouterErrorClockSkew:
-						s << " - Clock skew";
-					break;
-					case eRouterErrorOffline:
-						s << " - Offline";
-					break;
-					case eRouterErrorSymmetricNAT:
-						s << " - Symmetric NAT";
-					break;
-					default: ;
-				}
-				break;
-			}
 			default: s << "Unk";
+		};
+		if (testing)
+			s << " (Test)";
+		if (i2p::context.GetError () != eRouterErrorNone)
+		{
+			switch (i2p::context.GetError ())
+			{
+				case eRouterErrorClockSkew:
+					s << " - Clock skew";
+				break;
+				case eRouterErrorOffline:
+					s << " - Offline";
+				break;
+				case eRouterErrorSymmetricNAT:
+					s << " - Symmetric NAT";
+				break;
+				default: ;
+			}
 		}
 	}
 
@@ -180,11 +180,11 @@ namespace win32
 	{
 		s << "\n";
 		s << "Status: ";
-		ShowNetworkStatus (s, i2p::context.GetStatus ());
+		ShowNetworkStatus (s, i2p::context.GetStatus (), i2p::context.GetTesting ());
 		if (i2p::context.SupportsV6 ())
 		{
 			s << " / ";
-			ShowNetworkStatus (s, i2p::context.GetStatusV6 ());
+			ShowNetworkStatus (s, i2p::context.GetStatusV6 (), i2p::context.GetTestingV6 ());
 		}
 		s << "; ";
 		s << "Success Rate: " << i2p::tunnel::tunnels.GetTunnelCreationSuccessRate() << "%\n";
@@ -349,6 +349,9 @@ namespace win32
 						}
 					}
 				}
+#if (__cplusplus >= 201703L) // C++ 17 or higher
+				[[fallthrough]];
+#endif
 			}
 			case WM_TRAYICON:
 			{
@@ -418,8 +421,9 @@ namespace win32
 		return DefWindowProc( hWnd, uMsg, wParam, lParam);
 	}
 
-	bool StartWin32App ()
+	bool StartWin32App (bool isWinService)
 	{
+		g_isWinService = isWinService;
 		if (FindWindow (I2PD_WIN32_CLASSNAME, TEXT("i2pd")))
 		{
 			MessageBox(NULL, TEXT("I2Pd is running already"), TEXT("Warning"), MB_OK);
@@ -448,7 +452,9 @@ namespace win32
 			MessageBox(NULL, "Failed to create main window", TEXT("Warning!"), MB_ICONERROR | MB_OK | MB_TOPMOST);
 			return false;
 		}
-		SubscribeToEvents();
+		// COM requires message loop to work, which is not implemented in service mode
+		if (!g_isWinService)
+			SubscribeToEvents();
 		return true;
 	}
 
@@ -468,7 +474,8 @@ namespace win32
 		HWND hWnd = FindWindow (I2PD_WIN32_CLASSNAME, TEXT("i2pd"));
 		if (hWnd)
 			PostMessage (hWnd, WM_COMMAND, MAKEWPARAM(ID_EXIT, 0), 0);
-		// UnSubscribeFromEvents(); // TODO: understand why unsubscribing crashes app
+		else if(!g_isWinService)
+			UnSubscribeFromEvents();
 		UnregisterClass (I2PD_WIN32_CLASSNAME, GetModuleHandle(NULL));
 	}
 
