@@ -43,7 +43,7 @@ namespace data
 	RouterInfo::RouterInfo (const std::string& fullPath):
 		m_FamilyID (0), m_IsUpdated (false), m_IsUnreachable (false),
 		m_SupportedTransports (0),m_ReachableTransports (0),
-		m_Caps (0), m_Version (0)
+		m_Caps (0), m_Version (0), m_Congestion (eLowCongestion)
 	{
 		m_Addresses = boost::make_shared<Addresses>(); // create empty list
 		m_Buffer = NewBuffer (); // always RouterInfo's
@@ -53,7 +53,7 @@ namespace data
 	RouterInfo::RouterInfo (std::shared_ptr<Buffer>&& buf, size_t len):
 		m_FamilyID (0), m_IsUpdated (true), m_IsUnreachable (false),
 		m_SupportedTransports (0), m_ReachableTransports (0),
-		m_Caps (0), m_Version (0)
+		m_Caps (0), m_Version (0), m_Congestion (eLowCongestion)
 	{
 		if (len <= MAX_RI_BUFFER_SIZE)
 		{
@@ -79,13 +79,12 @@ namespace data
 	{
 	}
 
-	void RouterInfo::Update (const uint8_t * buf, size_t len)
+	bool RouterInfo::Update (const uint8_t * buf, size_t len)
 	{
 		if (len > MAX_RI_BUFFER_SIZE)
 		{
-			LogPrint (eLogError, "RouterInfo: Buffer is too long ", len);
-			m_IsUnreachable = true;
-			return;
+			LogPrint (eLogWarning, "RouterInfo: Updated buffer is too long ", len, ". Not changed");
+			return false;
 		}
 		// verify signature since we have identity already
 		int l = len - m_RouterIdentity->GetSignatureLen ();
@@ -99,20 +98,21 @@ namespace data
 			m_Caps = 0;
 			// don't clean up m_Addresses, it will be replaced in ReadFromStream
 			ClearProperties ();
-			// copy buffer
-			UpdateBuffer (buf, len);
 			// skip identity
 			size_t identityLen = m_RouterIdentity->GetFullLen ();
 			// read new RI
-			std::stringstream str (std::string ((char *)m_Buffer->data () + identityLen, m_BufferLen - identityLen));
+			std::stringstream str (std::string ((char *)buf + identityLen, len - identityLen));
 			ReadFromStream (str);
+			if (!m_IsUnreachable)
+				UpdateBuffer (buf, len); // save buffer	
 			// don't delete buffer until saved to the file
 		}
 		else
-		{
-			LogPrint (eLogError, "RouterInfo: Signature verification failed");
-			m_IsUnreachable = true;
-		}
+		{	
+			LogPrint (eLogWarning, "RouterInfo: Updated signature verification failed. Not changed");
+			return false;
+		}	
+		return true;
 	}
 
 	void RouterInfo::SetRouterIdentity (std::shared_ptr<const IdentityEx> identity)
@@ -161,7 +161,7 @@ namespace data
 			m_IsUnreachable = true;
 			return;
 		}
-		m_RouterIdentity = std::make_shared<IdentityEx>(m_Buffer->data (), m_BufferLen);
+		m_RouterIdentity = NewIdentity (m_Buffer->data (), m_BufferLen);
 		size_t identityLen = m_RouterIdentity->GetFullLen ();
 		if (identityLen >= m_BufferLen)
 		{
@@ -186,7 +186,6 @@ namespace data
 				m_IsUnreachable = true;
 				return;
 			}
-			m_RouterIdentity->DropVerifier ();
 		}
 		// parse RI
 		std::stringstream str;
@@ -202,7 +201,7 @@ namespace data
 	void RouterInfo::ReadFromStream (std::istream& s)
 	{
 		if (!s) return;
-		m_Caps = 0;
+		m_Caps = 0; m_Congestion = eLowCongestion;
 		s.read ((char *)&m_Timestamp, sizeof (m_Timestamp));
 		m_Timestamp = be64toh (m_Timestamp);
 		// read addresses
@@ -252,7 +251,15 @@ namespace data
 				{
 					boost::system::error_code ecode;
 					address->host = boost::asio::ip::address::from_string (value, ecode);
-					if (!ecode && !address->host.is_unspecified ()) isHost = true;
+					if (!ecode && !address->host.is_unspecified ())
+					{
+						if (!i2p::util::net::IsInReservedRange (address->host) ||
+						    i2p::util::net::IsYggdrasilAddress (address->host))
+							isHost = true;
+						else
+							// we consider such address as invalid
+							address->transportStyle = eTransportUnknown;
+					}
 				}
 				else if (!strcmp (key, "port"))
 				{
@@ -286,7 +293,8 @@ namespace data
 				else if (!strcmp (key, "s")) // ntcp2 or ssu2 static key
 				{
 					Base64ToByteStream (value, strlen (value), address->s, 32);
-					isStaticKey = true;
+					if (!(address->s[31] & 0x80)) // check if x25519 public key
+						isStaticKey = true;
 				}
 				else if (!strcmp (key, "i")) // ntcp2 iv or ssu2 intro
 				{
@@ -328,11 +336,7 @@ namespace data
 						address->ssu->introducers.resize (index + 1);
 					}
 					Introducer& introducer = address->ssu->introducers.at (index);
-					if (!strcmp (key, "ihost"))
-						introducer.isH = false; // SSU1
-					else if (!strcmp (key, "iport"))
-						introducer.isH = false; // SSU1
-					else if (!strcmp (key, "itag"))
+					if (!strcmp (key, "itag"))
 					{
 						try
 						{
@@ -344,10 +348,7 @@ namespace data
 						}
 					}
 					else if (!strcmp (key, "ih"))
-					{
 						Base64ToByteStream (value, strlen (value), introducer.iH, 32);
-						introducer.isH = true;
-					}
 					else if (!strcmp (key, "iexp"))
 					{
 						try
@@ -362,11 +363,12 @@ namespace data
 				}
 				if (!s) return;
 			}
+			
 			if (address->transportStyle == eTransportNTCP2)
 			{
 				if (isStaticKey)
 				{
-					if (isHost)
+					if (isHost && address->port)
 					{
 						if (address->host.is_v6 ())
 							supportedTransports |= (i2p::util::net::IsYggdrasilAddress (address->host) ? eNTCP2V6Mesh : eNTCP2V6);
@@ -374,8 +376,9 @@ namespace data
 							supportedTransports |= eNTCP2V4;
 						m_ReachableTransports |= supportedTransports;
 					}
-					else if (!address->published)
+					else
 					{
+						address->published = false;
 						if (address->caps)
 						{
 							if (address->caps & AddressCaps::eV4) supportedTransports |= eNTCP2V4;
@@ -386,11 +389,11 @@ namespace data
 					}
 				}
 			}
-			else if (address->transportStyle == eTransportSSU2 && isV2)
+			else if (address->transportStyle == eTransportSSU2 && isV2 && isStaticKey)
 			{
 				if (address->IsV4 ()) supportedTransports |= eSSU2V4;
 				if (address->IsV6 ()) supportedTransports |= eSSU2V6;
-				if (address->port)
+				if (isHost && address->port)
 				{
 					if (address->host.is_v4 ()) m_ReachableTransports |= eSSU2V4;
 					if (address->host.is_v6 ()) m_ReachableTransports |= eSSU2V6;
@@ -400,18 +403,9 @@ namespace data
 				{
 					// exclude invalid introducers
 					uint32_t ts = i2p::util::GetSecondsSinceEpoch ();
-					int numValid = 0;
-					for (auto& it: address->ssu->introducers)
-					{
-						if (it.iTag && ts < it.iExp && it.isH)
-							numValid++;
-						else
-							it.iTag = 0;
-					}
-					if (numValid)
+					UpdateIntroducers (address, ts);
+					if (!address->ssu->introducers.empty ()) // still has something
 						m_ReachableTransports |= supportedTransports;
-					else
-						address->ssu->introducers.resize (0);
 				}
 			}
 			if (supportedTransports)
@@ -491,7 +485,10 @@ namespace data
 				if (netdb.GetFamilies ().VerifyFamily (family, GetIdentHash (), value))
 					m_FamilyID = netdb.GetFamilies ().GetFamilyID (family);
 				else
+				{	
 					LogPrint (eLogWarning, "RouterInfo: Family ", family, " signature verification failed");
+					SetUnreachable (true);	
+				}		
 			}
 
 			if (!s) return;
@@ -534,6 +531,15 @@ namespace data
 				case CAPS_FLAG_UNREACHABLE:
 					m_Caps |= Caps::eUnreachable;
 				break;
+				case CAPS_FLAG_MEDIUM_CONGESTION:
+					m_Congestion = eMediumCongestion;
+				break;
+				case CAPS_FLAG_HIGH_CONGESTION:
+					m_Congestion = eHighCongestion;
+				break;
+				case CAPS_FLAG_REJECT_ALL_CONGESTION:
+					m_Congestion = eRejectAll;
+				break;	
 				default: ;
 			}
 			cap++;
@@ -567,6 +573,21 @@ namespace data
 		return caps;
 	}
 
+	void RouterInfo::UpdateIntroducers (std::shared_ptr<Address> address, uint64_t ts)
+	{
+		if (!address || !address->ssu) return;
+		int numValid = 0;
+		for (auto& it: address->ssu->introducers)
+		{
+			if (it.iTag && ts < it.iExp && !it.iH.IsZero ())
+				numValid++;
+			else
+				it.iTag = 0;
+		}
+		if (!numValid)
+			address->ssu->introducers.resize (0);
+	}	
+		
 	bool RouterInfo::IsNewer (const uint8_t * buf, size_t len) const
 	{
 		if (!m_RouterIdentity) return false;
@@ -581,20 +602,24 @@ namespace data
 		{
 			if (LoadFile (fullPath))
 				LogPrint (eLogDebug, "RouterInfo: Buffer for ", GetIdentHashAbbreviation (GetIdentHash ()), " loaded from file");
+			else
+				return nullptr;
 		}
 		return m_Buffer->data ();
 	}
 
 	bool RouterInfo::SaveToFile (const std::string& fullPath)
 	{
+		if (m_IsUnreachable) return false; // don't save bad router
 		if (!m_Buffer)
 		{
-			LogPrint (eLogError, "RouterInfo: Can't save, m_Buffer == NULL");
+			LogPrint (eLogWarning, "RouterInfo: Can't save, m_Buffer == NULL");
 			return false;
 		}
 		std::ofstream f (fullPath, std::ofstream::binary | std::ofstream::out);
-		if (!f.is_open ()) {
-			LogPrint(eLogError, "RouterInfo: Can't save to ", fullPath);
+		if (!f.is_open ()) 
+		{
+			LogPrint (eLogError, "RouterInfo: Can't save to ", fullPath);
 			return false;
 		}
 		f.write ((char *)m_Buffer->data (), m_BufferLen);
@@ -620,22 +645,49 @@ namespace data
 		return l+1;
 	}
 
-	void RouterInfo::AddNTCP2Address (const uint8_t * staticKey, const uint8_t * iv,
-		const boost::asio::ip::address& host, int port, uint8_t caps)
+	void RouterInfo::AddNTCP2Address (const uint8_t * staticKey, const uint8_t * iv,int port, uint8_t caps)
 	{
 		auto addr = std::make_shared<Address>();
-		addr->host = host;
 		addr->port = port;
 		addr->transportStyle = eTransportNTCP2;
 		addr->caps = caps;
 		addr->date = 0;
-		if (port) addr->published = true;
+		addr->published = false;
 		memcpy (addr->s, staticKey, 32);
 		memcpy (addr->i, iv, 16);
 		if (addr->IsV4 ())
 		{
 			m_SupportedTransports |= eNTCP2V4;
-			if (addr->published) m_ReachableTransports |= eNTCP2V4;
+			(*m_Addresses)[eNTCP2V4Idx] = addr;
+		}
+		if (addr->IsV6 ())
+		{
+			m_SupportedTransports |= eNTCP2V6;
+			(*m_Addresses)[eNTCP2V6Idx] = addr;
+		}
+	}
+
+	void RouterInfo::AddNTCP2Address (const uint8_t * staticKey, const uint8_t * iv,
+		const boost::asio::ip::address& host, int port)
+	{
+		auto addr = std::make_shared<Address>();
+		addr->host = host;
+		addr->port = port;
+		addr->transportStyle = eTransportNTCP2;
+		addr->date = 0;
+		addr->published = true;
+		memcpy (addr->s, staticKey, 32);
+		memcpy (addr->i, iv, 16);
+		addr->caps = 0;
+		if (host.is_unspecified ())
+		{	
+			if (host.is_v4 ()) addr->caps |= eV4;
+			if (host.is_v6 ()) addr->caps |= eV6;
+		}	
+		if (addr->IsV4 ())
+		{
+			m_SupportedTransports |= eNTCP2V4;
+			m_ReachableTransports |= eNTCP2V4;
 			(*m_Addresses)[eNTCP2V4Idx] = addr;
 		}
 		if (addr->IsV6 ())
@@ -649,17 +701,34 @@ namespace data
 			else
 			{
 				m_SupportedTransports |= eNTCP2V6;
-				if (addr->published) m_ReachableTransports |= eNTCP2V6;
+				m_ReachableTransports |= eNTCP2V6;
 				(*m_Addresses)[eNTCP2V6Idx] = addr;
 			}
 		}
 	}
 
-	void RouterInfo::AddSSU2Address (const uint8_t * staticKey, const uint8_t * introKey, uint8_t caps)
+	void RouterInfo::RemoveNTCP2Address (bool v4)
+	{
+		if (v4)
+		{
+			if ((*m_Addresses)[eNTCP2V6Idx])
+				(*m_Addresses)[eNTCP2V6Idx]->caps &= ~AddressCaps::eV4;
+			(*m_Addresses)[eNTCP2V4Idx].reset ();
+		}
+		else
+		{
+			if ((*m_Addresses)[eNTCP2V4Idx])
+				(*m_Addresses)[eNTCP2V4Idx]->caps &= ~AddressCaps::eV6;
+			(*m_Addresses)[eNTCP2V6Idx].reset ();
+		}
+		UpdateSupportedTransports ();
+	}
+
+	void RouterInfo::AddSSU2Address (const uint8_t * staticKey, const uint8_t * introKey, int port, uint8_t caps)
 	{
 		auto addr = std::make_shared<Address>();
 		addr->transportStyle = eTransportSSU2;
-		addr->port = 0;
+		addr->port = port;
 		addr->caps = caps;
 		addr->date = 0;
 		addr->ssu.reset (new SSUExt ());
@@ -686,12 +755,19 @@ namespace data
 		addr->host = host;
 		addr->port = port;
 		addr->published = true;
-		addr->caps = i2p::data::RouterInfo::eSSUTesting | i2p::data::RouterInfo::eSSUIntroducer; // BC;
 		addr->date = 0;
 		addr->ssu.reset (new SSUExt ());
 		addr->ssu->mtu = 0;
 		memcpy (addr->s, staticKey, 32);
 		memcpy (addr->i, introKey, 32);
+		if (!host.is_unspecified ())
+			addr->caps = i2p::data::RouterInfo::eSSUTesting | i2p::data::RouterInfo::eSSUIntroducer; // BC;
+		else
+		{	
+			addr->caps = 0;
+			if (host.is_v4 ()) addr->caps |= eV4;
+			if (host.is_v6 ()) addr->caps |= eV6;
+		}
 		if (addr->IsV4 ())
 		{
 			m_SupportedTransports |= eSSU2V4;
@@ -706,6 +782,23 @@ namespace data
 		}
 	}
 
+	void RouterInfo::RemoveSSU2Address (bool v4)
+	{
+		if (v4)
+		{
+			if ((*m_Addresses)[eSSU2V6Idx])
+				(*m_Addresses)[eSSU2V6Idx]->caps &= ~AddressCaps::eV4;
+			(*m_Addresses)[eSSU2V4Idx].reset ();
+		}
+		else
+		{
+			if ((*m_Addresses)[eSSU2V4Idx])
+				(*m_Addresses)[eSSU2V4Idx]->caps &= ~AddressCaps::eV6;
+			(*m_Addresses)[eSSU2V6Idx].reset ();
+		}
+		UpdateSupportedTransports ();
+	}
+
 	bool RouterInfo::IsNTCP2 (bool v4only) const
 	{
 		if (v4only)
@@ -744,14 +837,14 @@ namespace data
 		{
 			if ((*m_Addresses)[eNTCP2V6Idx])
 			{
-				if ((*m_Addresses)[eNTCP2V6Idx]->IsV4 ())
-					(*m_Addresses)[eNTCP2V6Idx]->caps &= ~AddressCaps::eV6;
+				if ((*m_Addresses)[eNTCP2V6Idx]->IsV4 () && (*m_Addresses)[eNTCP2V4Idx])
+					(*m_Addresses)[eNTCP2V4Idx]->caps &= ~AddressCaps::eV6;
 				(*m_Addresses)[eNTCP2V6Idx].reset ();
 			}
 			if ((*m_Addresses)[eSSU2V6Idx])
 			{
-				if ((*m_Addresses)[eSSU2V6Idx]->IsV4 ())
-					(*m_Addresses)[eSSU2V6Idx]->caps &= ~AddressCaps::eV6;
+				if ((*m_Addresses)[eSSU2V6Idx]->IsV4 () && (*m_Addresses)[eSSU2V4Idx])
+					(*m_Addresses)[eSSU2V4Idx]->caps &= ~AddressCaps::eV6;
 				(*m_Addresses)[eSSU2V6Idx].reset ();
 			}
 			UpdateSupportedTransports ();
@@ -764,14 +857,14 @@ namespace data
 		{
 			if ((*m_Addresses)[eNTCP2V4Idx])
 			{
-				if ((*m_Addresses)[eNTCP2V4Idx]->IsV6 ())
-					(*m_Addresses)[eNTCP2V4Idx]->caps &= ~AddressCaps::eV4;
+				if ((*m_Addresses)[eNTCP2V4Idx]->IsV6 () && (*m_Addresses)[eNTCP2V6Idx])
+					(*m_Addresses)[eNTCP2V6Idx]->caps &= ~AddressCaps::eV4;
 				(*m_Addresses)[eNTCP2V4Idx].reset ();
 			}
 			if ((*m_Addresses)[eSSU2V4Idx])
 			{
-				if ((*m_Addresses)[eSSU2V4Idx]->IsV6 ())
-					(*m_Addresses)[eSSU2V4Idx]->caps &= ~AddressCaps::eV4;
+				if ((*m_Addresses)[eSSU2V4Idx]->IsV6 () && (*m_Addresses)[eSSU2V6Idx])
+					(*m_Addresses)[eSSU2V6Idx]->caps &= ~AddressCaps::eV4;
 				(*m_Addresses)[eSSU2V4Idx].reset ();
 			}
 			UpdateSupportedTransports ();
@@ -846,23 +939,14 @@ namespace data
 		return nullptr;
 	}
 
-	std::shared_ptr<const RouterInfo::Address> RouterInfo::GetNTCP2AddressWithStaticKey (const uint8_t * key) const
+	std::shared_ptr<const RouterInfo::Address> RouterInfo::GetNTCP2V4Address () const
 	{
-		if (!key) return nullptr;
-		return GetAddress (
-			[key](std::shared_ptr<const RouterInfo::Address> address)->bool
-			{
-				return address->IsNTCP2 () && !memcmp (address->s, key, 32);
-			});
+		return (*GetAddresses ())[eNTCP2V4Idx];
 	}
 
-	std::shared_ptr<const RouterInfo::Address> RouterInfo::GetSSU2AddressWithStaticKey (const uint8_t * key, bool isV6) const
+	std::shared_ptr<const RouterInfo::Address> RouterInfo::GetNTCP2V6Address () const
 	{
-		if (!key) return nullptr;
-		auto addr = (*GetAddresses ())[isV6 ? eSSU2V6Idx : eSSU2V4Idx];
-		if (addr && !memcmp (addr->s, key, 32))
-		    return addr;
-		return nullptr;
+		return (*GetAddresses ())[eNTCP2V6Idx];
 	}
 
 	std::shared_ptr<const RouterInfo::Address> RouterInfo::GetPublishedNTCP2V4Address () const
@@ -886,9 +970,13 @@ namespace data
 
 	std::shared_ptr<RouterProfile> RouterInfo::GetProfile () const
 	{
-		if (!m_Profile)
-			m_Profile = GetRouterProfile (GetIdentHash ());
-		return m_Profile;
+		auto profile = m_Profile;
+		if (!profile)
+		{	
+			profile = GetRouterProfile (GetIdentHash ());
+			m_Profile = profile;
+		}	
+		return profile;
 	}
 
 	void RouterInfo::Encrypt (const uint8_t * data, uint8_t * encrypted) const
@@ -900,11 +988,23 @@ namespace data
 
 	bool RouterInfo::IsEligibleFloodfill () const
 	{
-		// floodfill must be reachable by ipv4, >= 0.9.38 and not DSA
-		return IsReachableBy (eNTCP2V4 | eSSU2V4) && m_Version >= NETDB_MIN_FLOODFILL_VERSION &&
+		// floodfill must have published ipv4, >= 0.9.38 and not DSA
+		return m_Version >= NETDB_MIN_FLOODFILL_VERSION && IsPublished (true) && 
 			GetIdentity ()->GetSigningKeyType () != SIGNING_KEY_TYPE_DSA_SHA1;
 	}
 
+	bool RouterInfo::IsPublished (bool v4) const
+	{
+		if (m_Caps & (eUnreachable | eHidden)) return false; // if router sets U or H we assume that all addreses are not published
+		auto addr = GetAddresses ();
+		if (v4)	
+			return ((*addr)[eNTCP2V4Idx] && ((*addr)[eNTCP2V4Idx])->published) ||
+				((*addr)[eSSU2V4Idx] && ((*addr)[eSSU2V4Idx])->published);
+		else
+			return ((*addr)[eNTCP2V6Idx] && ((*addr)[eNTCP2V6Idx])->published) ||
+				((*addr)[eSSU2V6Idx] && ((*addr)[eSSU2V6Idx])->published);
+	}	
+		
 	bool RouterInfo::IsSSU2PeerTesting (bool v4) const
 	{
 		if (!(m_SupportedTransports & (v4 ? eSSU2V4 : eSSU2V6))) return false;
@@ -960,6 +1060,31 @@ namespace data
 		}
 	}
 
+	void RouterInfo::UpdateIntroducers (uint64_t ts)
+	{
+		if (ts*1000 < m_Timestamp + INTRODUCER_UPDATE_INTERVAL) return;
+		if (m_ReachableTransports & eSSU2V4)
+		{
+			auto addr = (*GetAddresses ())[eSSU2V4Idx];
+			if (addr && addr->UsesIntroducer ())
+			{
+				UpdateIntroducers (addr, ts);
+				if (!addr->UsesIntroducer ()) // no more valid introducers
+					m_ReachableTransports &= ~eSSU2V4;
+			}	
+		}	
+		if (m_ReachableTransports & eSSU2V6)
+		{
+			auto addr = (*GetAddresses ())[eSSU2V6Idx];
+			if (addr && addr->UsesIntroducer ())
+			{
+				UpdateIntroducers (addr, ts);
+				if (!addr->UsesIntroducer ()) // no more valid introducers
+					m_ReachableTransports &= ~eSSU2V6;
+			}	
+		}	
+	}	
+		
 	void RouterInfo::UpdateBuffer (const uint8_t * buf, size_t len)
 	{
 		if (!m_Buffer)
@@ -977,11 +1102,16 @@ namespace data
 	std::shared_ptr<RouterInfo::Address> RouterInfo::NewAddress () const
 	{
 		return netdb.NewRouterInfoAddress ();
-	}	
-		
+	}
+
 	boost::shared_ptr<RouterInfo::Addresses> RouterInfo::NewAddresses () const
 	{
 		return netdb.NewRouterInfoAddresses ();
+	}
+
+	std::shared_ptr<IdentityEx> RouterInfo::NewIdentity (const uint8_t * buf, size_t len) const
+	{
+		return netdb.NewIdentity (buf, len);
 	}	
 		
 	void RouterInfo::RefreshTimestamp ()
@@ -989,6 +1119,27 @@ namespace data
 		m_Timestamp = i2p::util::GetMillisecondsSinceEpoch ();
 	}
 
+	bool RouterInfo::IsHighCongestion (bool highBandwidth) const
+	{
+		switch (m_Congestion)
+		{
+			case eLowCongestion:
+				return false;
+			break;
+			case eMediumCongestion:
+				return highBandwidth;
+			break;
+			case eHighCongestion:
+				return i2p::util::GetMillisecondsSinceEpoch () < m_Timestamp + HIGH_CONGESTION_INTERVAL*1000LL;
+			break;	
+			case eRejectAll:
+				return true;
+			break;	
+			default:
+				return false;
+		}
+	}
+		
 	void LocalRouterInfo::CreateBuffer (const PrivateKeys& privateKeys)
 	{
 		RefreshTimestamp ();
@@ -1040,9 +1191,34 @@ namespace data
 		if (c & eReachable) caps += CAPS_FLAG_REACHABLE; // reachable
 		if (c & eUnreachable) caps += CAPS_FLAG_UNREACHABLE; // unreachable
 
+		switch (GetCongestion ())
+		{
+			case eMediumCongestion:
+				caps += CAPS_FLAG_MEDIUM_CONGESTION;
+			break;	
+			case eHighCongestion:
+				caps += CAPS_FLAG_HIGH_CONGESTION;
+			break;		
+			case eRejectAll:
+				caps += CAPS_FLAG_REJECT_ALL_CONGESTION;
+			break;	
+			default: ;	
+		};	
+		
 		SetProperty ("caps", caps);
 	}
 
+	bool LocalRouterInfo::UpdateCongestion (Congestion c)
+	{
+		if (c != GetCongestion ())
+		{
+			SetCongestion (c);
+			UpdateCapsProperty ();
+			return true;
+		}	
+		return false;
+ 	}	
+		
 	void LocalRouterInfo::WriteToStream (std::ostream& s) const
 	{
 		auto addresses = GetAddresses ();
@@ -1079,44 +1255,37 @@ namespace data
 			s.write ((const char *)&cost, sizeof (cost));
 			s.write ((const char *)&address.date, sizeof (address.date));
 			std::stringstream properties;
-			bool isPublished = false;
+			bool isPublished = address.published && !address.host.is_unspecified () && address.port;
 			if (address.transportStyle == eTransportNTCP2)
 			{
-				if (address.IsNTCP2 ())
+				WriteString ("NTCP2", s);
+				// caps
+				if (!isPublished)
 				{
-					WriteString ("NTCP2", s);
-					if (address.IsPublishedNTCP2 () && !address.host.is_unspecified () && address.port)
-						isPublished = true;
-					else
-					{
-						WriteString ("caps", properties);
-						properties << '=';
-						std::string caps;
-						if (address.IsV4 ()) caps += CAPS_FLAG_V4;
-						if (address.IsV6 ()) caps += CAPS_FLAG_V6;
-						if (caps.empty ()) caps += CAPS_FLAG_V4;
-						WriteString (caps, properties);
-						properties << ';';
-					}
+					WriteString ("caps", properties);
+					properties << '=';
+					std::string caps;
+					if (address.IsV4 ()) caps += CAPS_FLAG_V4;
+					if (address.IsV6 () || address.host.is_v6 ()) caps += CAPS_FLAG_V6; // we set 6 for unspecified ipv6
+					if (caps.empty ()) caps += CAPS_FLAG_V4;
+					WriteString (caps, properties);
+					properties << ';';
 				}
-				else
-					continue; // don't write NTCP address
 			}
 			else if (address.transportStyle == eTransportSSU2)
 			{
 				WriteString ("SSU2", s);
 				// caps
 				std::string caps;
-				if (address.published)
+				if (isPublished)
 				{
-					isPublished = true;
 					if (address.IsPeerTesting ()) caps += CAPS_FLAG_SSU2_TESTING;
 					if (address.IsIntroducer ()) caps += CAPS_FLAG_SSU2_INTRODUCER;
 				}
 				else
 				{
 					if (address.IsV4 ()) caps += CAPS_FLAG_V4;
-					if (address.IsV6 ()) caps += CAPS_FLAG_V6;
+					if (address.IsV6 () || address.host.is_v6 ()) caps += CAPS_FLAG_V6; // we set 6 for unspecified ipv6
 					if (caps.empty ()) caps += CAPS_FLAG_V4;
 				}
 				if (!caps.empty ())
@@ -1152,6 +1321,7 @@ namespace data
 					int i = 0;
 					for (const auto& introducer: address.ssu->introducers)
 					{
+						if (!introducer.iTag) continue;
 						if (introducer.iExp) // expiration is specified
 						{
 							WriteString ("iexp" + boost::lexical_cast<std::string>(i), properties);
@@ -1164,6 +1334,7 @@ namespace data
 					i = 0;
 					for (const auto& introducer: address.ssu->introducers)
 					{
+						if (!introducer.iTag) continue;
 						WriteString ("ih" + boost::lexical_cast<std::string>(i), properties);
 						properties << '=';
 						char value[64];
@@ -1176,6 +1347,7 @@ namespace data
 					i = 0;
 					for (const auto& introducer: address.ssu->introducers)
 					{
+						if (!introducer.iTag) continue;
 						WriteString ("itag" + boost::lexical_cast<std::string>(i), properties);
 						properties << '=';
 						WriteString (boost::lexical_cast<std::string>(introducer.iTag), properties);
@@ -1269,10 +1441,15 @@ namespace data
 	{
 		return std::make_shared<Address> ();
 	}
-		
+
 	boost::shared_ptr<RouterInfo::Addresses> LocalRouterInfo::NewAddresses () const
 	{
 		return boost::make_shared<Addresses> ();
+	}
+
+	std::shared_ptr<IdentityEx> LocalRouterInfo::NewIdentity (const uint8_t * buf, size_t len) const
+	{
+		return std::make_shared<IdentityEx> (buf, len);
 	}	
 		
 	bool LocalRouterInfo::AddSSU2Introducer (const Introducer& introducer, bool v4)
