@@ -200,10 +200,10 @@ namespace transport
 							i2p::context.SetStatusV6 (eRouterStatusProxy);
 					}
 					else
-						LogPrint(eLogError, "Transports: Unsupported NTCP2 proxy URL ", ntcp2proxy);
+						LogPrint(eLogCritical, "Transports: Unsupported NTCP2 proxy URL ", ntcp2proxy);
 				}
 				else
-					LogPrint(eLogError, "Transports: Invalid NTCP2 proxy URL ", ntcp2proxy);
+					LogPrint(eLogCritical, "Transports: Invalid NTCP2 proxy URL ", ntcp2proxy);
 			}
 			else
 				m_NTCP2Server = new NTCP2Server ();
@@ -225,10 +225,10 @@ namespace transport
 							i2p::context.SetStatusV6 (eRouterStatusProxy);
 					}
 					else
-						LogPrint(eLogError, "Transports: Can't set SSU2 proxy ", ssu2proxy);
+						LogPrint(eLogCritical, "Transports: Can't set SSU2 proxy ", ssu2proxy);
 				}
 				else
-					LogPrint(eLogError, "Transports: Invalid SSU2 proxy URL ", ssu2proxy);
+					LogPrint(eLogCritical, "Transports: Invalid SSU2 proxy URL ", ssu2proxy);
 			}
 		}
 
@@ -458,8 +458,20 @@ namespace transport
 			it->second.sessions.front ()->SendI2NPMessages (msgs);
 		else
 		{
-			if (it->second.delayedMessages.size () < MAX_NUM_DELAYED_MESSAGES)
+			auto sz = it->second.delayedMessages.size (); 	
+			if (sz < MAX_NUM_DELAYED_MESSAGES)
 			{
+				if (sz < CHECK_PROFILE_NUM_DELAYED_MESSAGES && sz + msgs.size () >= CHECK_PROFILE_NUM_DELAYED_MESSAGES)
+				{
+					auto profile = i2p::data::GetRouterProfile (ident);
+					if (profile && profile->IsUnreachable ())
+					{
+						LogPrint (eLogWarning, "Transports: Peer profile for ", ident.ToBase64 (), " reports unreachable. Dropped");
+						std::unique_lock<std::mutex> l(m_PeersMutex);
+						m_Peers.erase (it);
+						return;
+					}	
+				}	
 				for (auto& it1: msgs)
 					it->second.delayedMessages.push_back (it1);
 			}
@@ -579,7 +591,7 @@ namespace transport
 			peer.router->GetCompatibleTransports (true);
 		peer.numAttempts = 0;
 		peer.priority.clear ();
-		bool ssu2 = rand () & 1;
+		bool ssu2 = peer.router->GetProfile ()->IsReal () ? (rand () & 1) : false; // try NTCP2 if router is not confirmed real
 		const auto& priority = ssu2 ? ssu2Priority : ntcp2Priority;
 		for (auto transport: priority)
 			if (transport & compatibleTransports)
@@ -638,8 +650,8 @@ namespace transport
 				auto router = i2p::data::netdb.GetRandomSSU2PeerTestRouter (true, excluded); // v4
 				if (router)
 				{
-					if (i2p::context.GetStatus () != eRouterStatusTesting)
-						i2p::context.SetStatus (eRouterStatusTesting);
+					if (!i2p::context.GetTesting ())
+						i2p::context.SetTesting (true);
 					m_SSU2Server->StartPeerTest (router, true);
 					excluded.insert (router->GetIdentHash ());
 				}
@@ -657,8 +669,8 @@ namespace transport
 				auto router = i2p::data::netdb.GetRandomSSU2PeerTestRouter (false, excluded); // v6
 				if (router)
 				{
-					if (i2p::context.GetStatusV6 () != eRouterStatusTesting)
-						i2p::context.SetStatusV6 (eRouterStatusTesting);
+					if (!i2p::context.GetTestingV6 ())
+						i2p::context.SetTestingV6 (true);
 					m_SSU2Server->StartPeerTest (router, false);
 					excluded.insert (router->GetIdentHash ());
 				}
@@ -688,6 +700,25 @@ namespace transport
 			auto it = m_Peers.find (ident);
 			if (it != m_Peers.end ())
 			{
+				if (it->second.numAttempts > 1)
+				{
+					// exclude failed transports
+					i2p::data::RouterInfo::CompatibleTransports transports = 0;
+					int numExcluded = it->second.numAttempts - 1;
+					if (numExcluded > (int)it->second.priority.size ()) numExcluded = it->second.priority.size ();
+					for (int i = 0; i < numExcluded; i++)
+						transports |= it->second.priority[i];
+					i2p::data::netdb.ExcludeReachableTransports (ident, transports);
+				}	
+				if (it->second.router && it->second.numAttempts)
+				{	
+					auto transport = it->second.priority[it->second.numAttempts-1];
+					if (transport == i2p::data::RouterInfo::eNTCP2V4 || 
+						transport == i2p::data::RouterInfo::eNTCP2V6 || transport == i2p::data::RouterInfo::eNTCP2V6Mesh)
+						it->second.router->GetProfile ()->Connected (); // outgoing NTCP2 connection if always real
+					i2p::data::netdb.SetUnreachable (ident, false); // clear unreachable 
+				}		
+				it->second.numAttempts = 0;
 				it->second.router = nullptr; // we don't need RouterInfo after successive connect
 				bool sendDatabaseStore = true;
 				if (it->second.delayedMessages.size () > 0)
@@ -706,7 +737,7 @@ namespace transport
 				session->SendI2NPMessages (it->second.delayedMessages);
 				it->second.delayedMessages.clear ();
 			}
-			else // incoming connection
+			else // incoming connection or peer test
 			{
 				if(RoutesRestricted() && ! IsRestrictedPeer(ident)) {
 					// not trusted
@@ -714,11 +745,15 @@ namespace transport
 					session->Done();
 					return;
 				}
-				session->SendI2NPMessages ({ CreateDatabaseStoreMsg () }); // send DatabaseStore
+				if (!session->IsOutgoing ()) // incoming
+					session->SendI2NPMessages ({ CreateDatabaseStoreMsg () }); // send DatabaseStore
+				auto r = i2p::data::netdb.FindRouter (ident); // router should be in netdb after SessionConfirmed
+				if (r) r->GetProfile ()->Connected ();
 				auto ts = i2p::util::GetSecondsSinceEpoch ();
 				std::unique_lock<std::mutex> l(m_PeersMutex);
-				auto it = m_Peers.insert (std::make_pair (ident, Peer{ nullptr, ts })).first;
+				auto it = m_Peers.insert (std::make_pair (ident, Peer{ r, ts })).first;
 				it->second.sessions.push_back (session);
+				it->second.router = nullptr;
 			}
 		});
 	}
@@ -775,11 +810,12 @@ namespace transport
  				if (it->second.sessions.empty () && ts > it->second.creationTime + SESSION_CREATION_TIMEOUT)
 				{
 					LogPrint (eLogWarning, "Transports: Session to peer ", it->first.ToBase64 (), " has not been created in ", SESSION_CREATION_TIMEOUT, " seconds");
-					auto profile = i2p::data::GetRouterProfile(it->first);
-					if (profile)
-					{
-						profile->TunnelNonReplied();
-					}
+				/*	if (!it->second.router) 
+					{	 
+						// if router for ident not found mark it unreachable
+						auto profile = i2p::data::GetRouterProfile (it->first);
+						if (profile) profile->Unreachable ();
+					}	*/
 					std::unique_lock<std::mutex> l(m_PeersMutex);
 					it = m_Peers.erase (it);
 				}
@@ -796,8 +832,8 @@ namespace transport
 					++it;
 				}
 			}
-			bool ipv4Testing = i2p::context.GetStatus () == eRouterStatusTesting;
-			bool ipv6Testing = i2p::context.GetStatusV6 () == eRouterStatusTesting;
+			bool ipv4Testing = i2p::context.GetTesting ();
+			bool ipv6Testing = i2p::context.GetTestingV6 ();
 			// if still testing, repeat peer test
 			if (ipv4Testing || ipv6Testing)
 				PeerTest (ipv4Testing, ipv6Testing);
@@ -826,7 +862,9 @@ namespace transport
 			uint16_t inds[3];
 			RAND_bytes ((uint8_t *)inds, sizeof (inds));
 			std::unique_lock<std::mutex> l(m_PeersMutex);
-			inds[0] %= m_Peers.size ();
+			auto count = m_Peers.size ();
+			if(count == 0) return nullptr;
+			inds[0] %= count;
 			auto it = m_Peers.begin ();
 			std::advance (it, inds[0]);
 			// try random peer
@@ -906,9 +944,10 @@ namespace transport
 		return GetRandomPeer (
 			[isHighBandwidth](const Peer& peer)->bool
 			{
-				// connected and not overloaded
-				return !peer.router && !peer.sessions.empty () &&
+				// connected, not overloaded and not slow
+				return !peer.router && !peer.sessions.empty () && peer.isReachable &&
 					peer.sessions.front ()->GetSendQueueSize () <= PEER_ROUTER_INFO_OVERLOAD_QUEUE_SIZE &&
+					!peer.sessions.front ()->IsSlow () &&
 					(!isHighBandwidth || peer.isHighBandwidth);
 			});
 	}
