@@ -41,6 +41,7 @@ namespace tunnel
 	const int MAX_NUM_RECORDS = 8;
 	const int HIGH_LATENCY_PER_HOP = 250; // in milliseconds
 	const int MAX_TUNNEL_MSGS_BATCH_SIZE = 100; // handle messages without interrupt
+	const uint16_t DEFAULT_MAX_NUM_TRANSIT_TUNNELS = 5000;
 	const int TUNNEL_MANAGE_INTERVAL = 15; // in seconds
 	const int TUNNEL_POOLS_MANAGE_INTERVAL = 5; // in seconds
 	const int TUNNEL_MEMORY_POOL_MANAGE_INTERVAL = 120; // in seconds
@@ -48,6 +49,9 @@ namespace tunnel
 	const size_t I2NP_TUNNEL_MESSAGE_SIZE = TUNNEL_DATA_MSG_SIZE + I2NP_HEADER_SIZE + 34; // reserved for alignment and NTCP 16 + 6 + 12
 	const size_t I2NP_TUNNEL_ENPOINT_MESSAGE_SIZE = 2*TUNNEL_DATA_MSG_SIZE + I2NP_HEADER_SIZE + TUNNEL_GATEWAY_HEADER_SIZE + 28; // reserved for alignment and NTCP 16 + 6 + 6
 
+	const double TCSR_SMOOTHING_CONSTANT = 0.0005; // smoothing constant in exponentially weighted moving average
+	const double TCSR_START_VALUE = 0.1; // start value of tunnel creation success rate
+
 	enum TunnelState
 	{
 		eTunnelStatePending,
@@ -99,8 +103,8 @@ namespace tunnel
 			bool HandleTunnelBuildResponse (uint8_t * msg, size_t len);
 
 			// implements TunnelBase
-			void SendTunnelDataMsg (std::shared_ptr<i2p::I2NPMessage> msg);
-			void EncryptTunnelMsg (std::shared_ptr<const I2NPMessage> in, std::shared_ptr<I2NPMessage> out);
+			void SendTunnelDataMsg (std::shared_ptr<i2p::I2NPMessage> msg) override;
+			void EncryptTunnelMsg (std::shared_ptr<const I2NPMessage> in, std::shared_ptr<I2NPMessage> out) override;
 
 			/** @brief add latency sample */
 			void AddLatencySample(const uint64_t ms) { m_Latency = (m_Latency + ms) >> 1; }
@@ -134,15 +138,15 @@ namespace tunnel
 			OutboundTunnel (std::shared_ptr<const TunnelConfig> config):
 				Tunnel (config), m_Gateway (this), m_EndpointIdentHash (config->GetLastIdentHash ()) {};
 
-			void SendTunnelDataMsg (const uint8_t * gwHash, uint32_t gwTunnel, std::shared_ptr<i2p::I2NPMessage> msg);
-			virtual void SendTunnelDataMsg (const std::vector<TunnelMessageBlock>& msgs); // multiple messages
+			void SendTunnelDataMsgTo (const uint8_t * gwHash, uint32_t gwTunnel, std::shared_ptr<i2p::I2NPMessage> msg);
+			virtual void SendTunnelDataMsgs (const std::vector<TunnelMessageBlock>& msgs); // multiple messages
 			const i2p::data::IdentHash& GetEndpointIdentHash () const { return m_EndpointIdentHash; };
 			virtual size_t GetNumSentBytes () const { return m_Gateway.GetNumSentBytes (); };
 
 			// implements TunnelBase
-			void HandleTunnelDataMsg (std::shared_ptr<i2p::I2NPMessage>&& tunnelMsg);
+			void HandleTunnelDataMsg (std::shared_ptr<i2p::I2NPMessage>&& tunnelMsg) override;
 
-			bool IsInbound() const { return false; }
+			bool IsInbound() const override { return false; }
 
 		private:
 
@@ -156,12 +160,12 @@ namespace tunnel
 		public:
 
 			InboundTunnel (std::shared_ptr<const TunnelConfig> config): Tunnel (config), m_Endpoint (true) {};
-			void HandleTunnelDataMsg (std::shared_ptr<I2NPMessage>&& msg);
+			void HandleTunnelDataMsg (std::shared_ptr<I2NPMessage>&& msg) override;
 			virtual size_t GetNumReceivedBytes () const { return m_Endpoint.GetNumReceivedBytes (); };
-			bool IsInbound() const { return true; }
+			bool IsInbound() const override { return true; }
 
 			// override TunnelBase
-			void Cleanup () { m_Endpoint.Cleanup (); };
+			void Cleanup () override { m_Endpoint.Cleanup (); };
 
 		private:
 
@@ -173,8 +177,8 @@ namespace tunnel
 		public:
 
 			ZeroHopsInboundTunnel ();
-			void SendTunnelDataMsg (std::shared_ptr<i2p::I2NPMessage> msg);
-			size_t GetNumReceivedBytes () const { return m_NumReceivedBytes; };
+			void SendTunnelDataMsg (std::shared_ptr<i2p::I2NPMessage> msg) override;
+			size_t GetNumReceivedBytes () const override { return m_NumReceivedBytes; };
 
 		private:
 
@@ -186,8 +190,8 @@ namespace tunnel
 		public:
 
 			ZeroHopsOutboundTunnel ();
-			void SendTunnelDataMsg (const std::vector<TunnelMessageBlock>& msgs);
-			size_t GetNumSentBytes () const { return m_NumSentBytes; };
+			void SendTunnelDataMsgs (const std::vector<TunnelMessageBlock>& msgs) override;
+			size_t GetNumSentBytes () const override { return m_NumSentBytes; };
 
 		private:
 
@@ -226,6 +230,10 @@ namespace tunnel
 
 			std::shared_ptr<I2NPMessage> NewI2NPTunnelMessage (bool endpoint);
 
+			void SetMaxNumTransitTunnels (uint16_t maxNumTransitTunnels);
+			uint16_t GetMaxNumTransitTunnels () const { return m_MaxNumTransitTunnels; };
+			bool IsTooManyTransitTunnels () const { return m_TransitTunnels.size () >= m_MaxNumTransitTunnels; };
+
 		private:
 
 			template<class TTunnel>
@@ -238,18 +246,36 @@ namespace tunnel
 			void HandleTunnelGatewayMsg (std::shared_ptr<TunnelBase> tunnel, std::shared_ptr<I2NPMessage> msg);
 
 			void Run ();
-			void ManageTunnels ();
-			void ManageOutboundTunnels ();
-			void ManageInboundTunnels ();
-			void ManageTransitTunnels ();
-			void ManagePendingTunnels ();
+			void ManageTunnels (uint64_t ts);
+			void ManageOutboundTunnels (uint64_t ts);
+			void ManageInboundTunnels (uint64_t ts);
+			void ManageTransitTunnels (uint64_t ts);
+			void ManagePendingTunnels (uint64_t ts);
 			template<class PendingTunnels>
-			void ManagePendingTunnels (PendingTunnels& pendingTunnels);
+			void ManagePendingTunnels (PendingTunnels& pendingTunnels, uint64_t ts);
 			void ManageTunnelPools (uint64_t ts);
 
 			std::shared_ptr<ZeroHopsInboundTunnel> CreateZeroHopsInboundTunnel (std::shared_ptr<TunnelPool> pool);
 			std::shared_ptr<ZeroHopsOutboundTunnel> CreateZeroHopsOutboundTunnel (std::shared_ptr<TunnelPool> pool);
 
+			// Calculating of tunnel creation success rate
+			void SuccesiveTunnelCreation()
+			{
+				// total TCSR
+				m_TotalNumSuccesiveTunnelCreations++;
+				// A modified version of the EWMA algorithm, where alpha is increased at the beginning to accelerate similarity
+				double alpha = TCSR_SMOOTHING_CONSTANT + (1 - TCSR_SMOOTHING_CONSTANT)/++m_TunnelCreationAttemptsNum;
+				m_TunnelCreationSuccessRate = alpha * 1 + (1 - alpha) * m_TunnelCreationSuccessRate;
+
+			}
+			void FailedTunnelCreation()
+			{
+				m_TotalNumFailedTunnelCreations++;
+
+				double alpha = TCSR_SMOOTHING_CONSTANT + (1 - TCSR_SMOOTHING_CONSTANT)/++m_TunnelCreationAttemptsNum;
+				m_TunnelCreationSuccessRate = alpha * 0 + (1 - alpha) * m_TunnelCreationSuccessRate;
+			}
+
 		private:
 
 			bool m_IsRunning;
@@ -266,9 +292,11 @@ namespace tunnel
 			i2p::util::Queue<std::shared_ptr<I2NPMessage> > m_Queue;
 			i2p::util::MemoryPoolMt<I2NPMessageBuffer<I2NP_TUNNEL_ENPOINT_MESSAGE_SIZE> > m_I2NPTunnelEndpointMessagesMemoryPool;
 			i2p::util::MemoryPoolMt<I2NPMessageBuffer<I2NP_TUNNEL_MESSAGE_SIZE> > m_I2NPTunnelMessagesMemoryPool;
-
-			// some stats
-			int m_NumSuccesiveTunnelCreations, m_NumFailedTunnelCreations;
+			uint16_t m_MaxNumTransitTunnels;
+			// count of tunnels for total TCSR algorithm
+			int m_TotalNumSuccesiveTunnelCreations, m_TotalNumFailedTunnelCreations;
+			double m_TunnelCreationSuccessRate;
+			int m_TunnelCreationAttemptsNum;
 
 		public:
 
@@ -282,10 +310,12 @@ namespace tunnel
 			size_t CountOutboundTunnels() const;
 
 			int GetQueueSize () { return m_Queue.GetSize (); };
-			int GetTunnelCreationSuccessRate () const // in percents
+			int GetTunnelCreationSuccessRate () const { return std::round(m_TunnelCreationSuccessRate * 100); } // in percents
+			double GetPreciseTunnelCreationSuccessRate () const { return m_TunnelCreationSuccessRate * 100; } // in percents
+			int GetTotalTunnelCreationSuccessRate () const // in percents
 			{
-				int totalNum = m_NumSuccesiveTunnelCreations + m_NumFailedTunnelCreations;
-				return totalNum ? m_NumSuccesiveTunnelCreations*100/totalNum : 0;
+				int totalNum = m_TotalNumSuccesiveTunnelCreations + m_TotalNumFailedTunnelCreations;
+				return totalNum ? m_TotalNumSuccesiveTunnelCreations*100/totalNum : 0;
 			}
 	};
 
